// White triangle with rotation used for demonstration of glutWarpPointer.
// 2020: Now tested under Linux! Only an adjusted include was needed.

// Old command-line:
// gcc warprotation.c ../common/*.c ../common/Linux/*.c -lGL -o warprotation -I../common -I../common/Linux -DGL_GLEXT_PROTOTYPES  -lXt -lX11 -lm

// C++ version changed nothing but the use of the latest version does matter.

	// uses framework Cocoa
	// uses framework OpenGL
#define MAIN
#include "MicroGlut.h"
#include "GL_utilities.h"
#include "VectorUtils4.h"

// Globals
// Data would normally be read from files
GLfloat vertices[] =
					{	-0.5f,-0.5f,0.0f,
						-0.5f,0.5f,0.0f,
						0.5f,-0.5f,0.0f };

mat4 myMatrix;
float a;

// vertex array object
unsigned int vertexArrayObjID;
	// Reference to shader program
	GLuint program;

void init(void)
{
	// two vertex buffer objects, used for uploading the
	unsigned int vertexBufferObjID;

	// GL inits
	glClearColor(0.2,0.2,0.5,0);
	glEnable(GL_DEPTH_TEST);
	printError("GL inits");

	// Load and compile shader
	program = loadShaders("rotation.vert", "rotation.frag");
	glUseProgram(program);
	printError("init shader");
	
	// Upload geometry to the GPU:
	
	// Allocate and activate Vertex Array Object
	glGenVertexArrays(1, &vertexArrayObjID);
	glBindVertexArray(vertexArrayObjID);
	// Allocate Vertex Buffer Objects
	glGenBuffers(1, &vertexBufferObjID);
	
	// VBO for vertex data
	glBindBuffer(GL_ARRAY_BUFFER, vertexBufferObjID);
	glBufferData(GL_ARRAY_BUFFER, 9*sizeof(GLfloat), vertices, GL_STATIC_DRAW);
	glVertexAttribPointer(glGetAttribLocation(program, "in_Position"), 3, GL_FLOAT, GL_FALSE, 0, 0); 
	glEnableVertexAttribArray(glGetAttribLocation(program, "in_Position"));

	// End of upload of geometry

// Upload the rotation matrix
	myMatrix = IdentityMatrix();
	glUniformMatrix4fv(glGetUniformLocation(program, "myMatrix"), 1, GL_TRUE, myMatrix.m);
	
	printError("init arrays");
}


void display(void)
{
	printError("pre display");

	// clear the screen
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	myMatrix = Rz(a);
	glUniformMatrix4fv(glGetUniformLocation(program, "myMatrix"), 1, GL_TRUE, myMatrix.m);

	glBindVertexArray(vertexArrayObjID);	// Select VAO
	glDrawArrays(GL_TRIANGLES, 0, 3);	// draw object
	
	printError("display");
	
	glutSwapBuffers();
}

void mouse(int x, int y)
{
	if (x != 100 || y != 100)
	{
		printf("You are at %d %d\n", x, y); // Window local!
		a += ((float)x - 100.0) / 100.0;
		printf("a =  %f\n", a);
		glutWarpPointer(100, 100);
		glutPostRedisplay();
	}
}

void key(unsigned char key, int x, int y)
{
	exit(0);
}

int main(int argc, char *argv[])
{
	glutInit(&argc, argv);
	glutInitContextVersion(3, 2);
	glutCreateWindow ("GL3 glutWarpPointer example");
	glutDisplayFunc(display); 
	glutPassiveMotionFunc(mouse);
	glutKeyboardFunc(key);
	glutHideCursor();
	init ();
	glutMainLoop();
}
